#include "memorystring.h"

#include <cstring>

InMemoryString::InMemoryString( char *beg,  char *end):beg(beg),end(end)
{
}

InMemoryString::InMemoryString(const InMemoryString& copy):beg(copy.beg),end(copy.end),ownptr(copy.ownptr)
{

}

InMemoryString& InMemoryString::operator=(const InMemoryString& copy)
{
    if(this == &copy)
        return *this;
    beg = copy.beg;
    end = copy.end;
    ownptr = copy.ownptr;
    return *this;
}

void InMemoryString::ownMemory()
{
    if(ownptr)
        return;
    size_t size = end - beg;
    char* begin = new char[size];
    std::memcpy(begin,beg,size);
    beg = begin;
    end = beg + size;
    ownptr.reset(beg,array_deleter<char>());
}

bool operator==(const InMemoryString& lhv, const InMemoryString& rhv)
{
    if((lhv.end - lhv.beg) != (rhv.end - rhv.beg))
        return false;
    char* lhv_it = lhv.beg;
    char* rhv_it = rhv.beg;
    while(lhv_it != lhv.end && rhv_it != rhv.end)
    {
        if(lhv_it && rhv_it && tolower(*lhv_it) != tolower(*rhv_it))
                return false;
        ++lhv_it;
        ++rhv_it;
    }
    return true;
}

bool operator!=(const InMemoryString& lhv, const InMemoryString& rhv)
{
    return !(lhv == rhv);
}

bool operator>(const InMemoryString& lhv, const InMemoryString& rhv)
{
    char* lhv_it = lhv.beg;
    char* rhv_it = rhv.beg;
    while(lhv_it != lhv.end && rhv_it != rhv.end)
    {
        if(lhv_it && rhv_it && (tolower(*lhv_it) > tolower(*rhv_it)))
                return true;
        ++lhv_it;
        ++rhv_it;
    }
    return false;
}

bool operator<(const InMemoryString& lhv, const InMemoryString& rhv)
{
    return !(lhv>rhv);
}

char InMemoryString::operator[](const size_t index)const
{
    return beg[index];
}
