#ifndef MEMORYSTRING_H
#define MEMORYSTRING_H

#include <string>
#include <memory>
#include <iostream>
#include <cctype>

class MemHash;

template<class T>
struct array_deleter
{
    void operator()(T* p)
    {
        delete[] p;
    }
};

class InMemoryString
{
    char* beg,*end;
    std::shared_ptr<char> ownptr;
    friend MemHash;
public:
    InMemoryString(char *beg, char *end);
    InMemoryString(const InMemoryString& copy);
    InMemoryString& operator=(const InMemoryString& copy);
    void ownMemory();


    std::string toString()const
    {
        if(beg && end)
            return std::string(beg,end);
        return std::string();
    }
    size_t size()const{return end - beg;}

    char operator[](const size_t index)const;
    friend bool operator==(const InMemoryString& lhv, const InMemoryString& rhv);
    friend bool operator!=(const InMemoryString& lhv, const InMemoryString& rhv);
    friend bool operator>(const InMemoryString& lhv, const InMemoryString& rhv);
    friend bool operator<(const InMemoryString& lhv, const InMemoryString& rhv);
};

struct MemHash
{
    size_t operator()(const InMemoryString& str)const
    {
        size_t result = 216613621U;
        char* end = str.end;
        for(char* it = str.beg; it != end;++it)
            result = 127 * result + static_cast<unsigned char>(tolower(*it));
        return result;
    }
    bool operator()( const InMemoryString& lhv,const InMemoryString& rhv)const
    {
        return (*this)(lhv)==(*this)(rhv);
    }
};

#endif // MEMORYSTRING_H
