#include "wordcounter.h"

#include <algorithm>
#include <fstream>
#include <iostream>
#include <memory>
#include <cctype>

WordCounter::WordCounter()
{
}


WordCounter::WordCounter(const WordCounter& copy):wordcount(copy.wordcount)
{

}

WordCounter& WordCounter::operator=(const WordCounter& copy)
{
    if(this == &copy)
        return *this;
    // initialize data
    wordcount = copy.wordcount;
    return *this;
}

WordCounter::~WordCounter()
{

}

void WordCounter::countWordsThreaded(const std::string file)
{
    std::lock_guard<std::mutex> lock(mymutex);
    isrunning = true;
    std::ifstream in(file);
    if(!in)
    {
        return;
    }
    in.seekg(0,std::ios::end);
    auto size = std::streamoff(in.tellg());
    if(size < 1)
        return;
    in.seekg(0,std::ios::beg);

    data.reset(new char[size]);
    in.read(data.get(),size);

    //wordcontainer& words=wordcount;

    for(char* it= data.get(),* end = data.get()+size,*lastwordbegin = nullptr;it != end;++it)
    {
        if(lastwordbegin && !isWord(*it) )
        {
#ifndef USE_INMEMORY
            ++wordcount[std::string(lastwordbegin,it)];
#else
            ++wordcount[InMemoryString(lastwordbegin,it)];
#endif
            lastwordbegin=nullptr;
        }
        else if(!lastwordbegin && isWord(*it))
            lastwordbegin = it;
    }
    //wordcount.swap(words);
    isrunning = false;
}

void WordCounter::startCounting(const std::string file)
{
    //std::cout << "startCounting " << file << std::endl;
    if(mythread && mythread->joinable())
    {
        std::cout << "thread still running" << std::endl;
        return;
    }
    mythread.reset( new std::thread(&WordCounter::countWordsThreaded,this, file));
}

void WordCounter::copyWords(wordcontainer &words)
{
    if(!mythread)
        return;
    if( isRunning())
        mythread->join();//wait for finish
    wordcontainer::iterator it;
#ifndef USE_INMEMORY
    for(std::pair<const std::string,size_t>& pair: wordcount)
#else
    for(std::pair<const InMemoryString,size_t>& pair: wordcount)
#endif
    {
        it = words.find(pair.first);
        if(it != words.end())
            it->second += pair.second;
        else
        {
#ifdef USE_INMEMORY
            InMemoryString* str = const_cast<InMemoryString* >(&(pair.first));
            str->ownMemory();
#endif
            words[pair.first] = pair.second;
        }
        pair.second = 0;

    }
    //wordcount.clear();
}

bool WordCounter::isRunning()const
{
    std::lock_guard<std::mutex> lock(mymutex);
    return isrunning;
}

bool WordCounter::isWord(const char c)
{
    return std::isalnum(c) || c == '_';
}

void WordCounter::join()
{
    if(mythread)
        mythread->join();
}
